package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickIMUActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickIMUGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickIMU.Identity value = ((BrickIMU)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickIMUGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickIMUActions) {
            return ((BrickIMUActions) actions).brickIMUGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickIMUActions class.");
        }
    }

    @RuleAction(label = "Is Status LED Enabled")
    public @ActionOutput(name = "enabled", type="boolean")
           Map<String, Object> brickIMUIsStatusLEDEnabled(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickIMU)this.handler.getDevice()).isStatusLEDEnabled();
        result.put("enabled", value);
        return result;
    }

    public static Map<String, Object> brickIMUIsStatusLEDEnabled(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickIMUActions) {
            return ((BrickIMUActions) actions).brickIMUIsStatusLEDEnabled();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickIMUActions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickIMUGetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickIMU)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickIMUGetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickIMUActions) {
            return ((BrickIMUActions) actions).brickIMUGetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickIMUActions class.");
        }
    }

    @RuleAction(label = "Get Send Timeout Count")
    public @ActionOutput(name = "timeoutCount", type="long")
           Map<String, Object> brickIMUGetSendTimeoutCount(
            @ActionInput(name = "communicationMethod") int communicationMethod) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickIMU)this.handler.getDevice()).getSendTimeoutCount(communicationMethod);
        result.put("timeoutCount", value);
        return result;
    }

    public static Map<String, Object> brickIMUGetSendTimeoutCount(@Nullable ThingActions actions, int communicationMethod) throws TinkerforgeException {
        if (actions instanceof BrickIMUActions) {
            return ((BrickIMUActions) actions).brickIMUGetSendTimeoutCount(communicationMethod);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickIMUActions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Baudrate Config")
    public @ActionOutput(name = "enableDynamicBaudrate", type="boolean")
           @ActionOutput(name = "minimumDynamicBaudrate", type="long")
           Map<String, Object> brickIMUGetSPITFPBaudrateConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickIMU.SPITFPBaudrateConfig value = ((BrickIMU)this.handler.getDevice()).getSPITFPBaudrateConfig();
        result.put("enableDynamicBaudrate", value.enableDynamicBaudrate);
        result.put("minimumDynamicBaudrate", value.minimumDynamicBaudrate);
        return result;
    }

    public static Map<String, Object> brickIMUGetSPITFPBaudrateConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickIMUActions) {
            return ((BrickIMUActions) actions).brickIMUGetSPITFPBaudrateConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickIMUActions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountACKChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickIMUGetSPITFPErrorCount(
            @ActionInput(name = "brickletPort") char brickletPort) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickIMU.SPITFPErrorCount value = ((BrickIMU)this.handler.getDevice()).getSPITFPErrorCount(brickletPort);
        result.put("errorCountACKChecksum", value.errorCountACKChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickIMUGetSPITFPErrorCount(@Nullable ThingActions actions, char brickletPort) throws TinkerforgeException {
        if (actions instanceof BrickIMUActions) {
            return ((BrickIMUActions) actions).brickIMUGetSPITFPErrorCount(brickletPort);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickIMUActions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Baudrate")
    public @ActionOutput(name = "baudrate", type="long")
           Map<String, Object> brickIMUGetSPITFPBaudrate(
            @ActionInput(name = "brickletPort") char brickletPort) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickIMU)this.handler.getDevice()).getSPITFPBaudrate(brickletPort);
        result.put("baudrate", value);
        return result;
    }

    public static Map<String, Object> brickIMUGetSPITFPBaudrate(@Nullable ThingActions actions, char brickletPort) throws TinkerforgeException {
        if (actions instanceof BrickIMUActions) {
            return ((BrickIMUActions) actions).brickIMUGetSPITFPBaudrate(brickletPort);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickIMUActions class.");
        }
    }
}
