package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickREDActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Create Session")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "sessionId", type="int")
           Map<String, Object> brickREDCreateSession(
            @ActionInput(name = "lifetime") long lifetime) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.CreateSession value = ((BrickRED)this.handler.getDevice()).createSession(lifetime);
        result.put("errorCode", value.errorCode);
        result.put("sessionId", value.sessionId);
        return result;
    }

    public static Map<String, Object> brickREDCreateSession(@Nullable ThingActions actions, long lifetime) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDCreateSession(lifetime);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Expire Session")
    public @ActionOutput(name = "errorCode", type="int")
           Map<String, Object> brickREDExpireSession(
            @ActionInput(name = "sessionId") int sessionId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickRED)this.handler.getDevice()).expireSession(sessionId);
        result.put("errorCode", value);
        return result;
    }

    public static Map<String, Object> brickREDExpireSession(@Nullable ThingActions actions, int sessionId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDExpireSession(sessionId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Expire Session Unchecked")
    public void brickREDExpireSessionUnchecked(
            @ActionInput(name = "sessionId") int sessionId) throws TinkerforgeException {
        ((BrickRED)this.handler.getDevice()).expireSessionUnchecked(sessionId);
    }

    public static void brickREDExpireSessionUnchecked(@Nullable ThingActions actions, int sessionId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            ((BrickREDActions) actions).brickREDExpireSessionUnchecked(sessionId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Keep Session Alive")
    public @ActionOutput(name = "errorCode", type="int")
           Map<String, Object> brickREDKeepSessionAlive(
            @ActionInput(name = "sessionId") int sessionId,
            @ActionInput(name = "lifetime") long lifetime) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickRED)this.handler.getDevice()).keepSessionAlive(sessionId, lifetime);
        result.put("errorCode", value);
        return result;
    }

    public static Map<String, Object> brickREDKeepSessionAlive(@Nullable ThingActions actions, int sessionId, long lifetime) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDKeepSessionAlive(sessionId, lifetime);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Release Object")
    public @ActionOutput(name = "errorCode", type="int")
           Map<String, Object> brickREDReleaseObject(
            @ActionInput(name = "objectId") int objectId,
            @ActionInput(name = "sessionId") int sessionId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickRED)this.handler.getDevice()).releaseObject(objectId, sessionId);
        result.put("errorCode", value);
        return result;
    }

    public static Map<String, Object> brickREDReleaseObject(@Nullable ThingActions actions, int objectId, int sessionId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDReleaseObject(objectId, sessionId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Release Object Unchecked")
    public void brickREDReleaseObjectUnchecked(
            @ActionInput(name = "objectId") int objectId,
            @ActionInput(name = "sessionId") int sessionId) throws TinkerforgeException {
        ((BrickRED)this.handler.getDevice()).releaseObjectUnchecked(objectId, sessionId);
    }

    public static void brickREDReleaseObjectUnchecked(@Nullable ThingActions actions, int objectId, int sessionId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            ((BrickREDActions) actions).brickREDReleaseObjectUnchecked(objectId, sessionId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Allocate String")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "stringId", type="int")
           Map<String, Object> brickREDAllocateString(
            @ActionInput(name = "lengthToReserve") long lengthToReserve,
            @ActionInput(name = "buffer") String buffer,
            @ActionInput(name = "sessionId") int sessionId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.AllocateString value = ((BrickRED)this.handler.getDevice()).allocateString(lengthToReserve, buffer, sessionId);
        result.put("errorCode", value.errorCode);
        result.put("stringId", value.stringId);
        return result;
    }

    public static Map<String, Object> brickREDAllocateString(@Nullable ThingActions actions, long lengthToReserve, String buffer, int sessionId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDAllocateString(lengthToReserve, buffer, sessionId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Truncate String")
    public @ActionOutput(name = "errorCode", type="int")
           Map<String, Object> brickREDTruncateString(
            @ActionInput(name = "stringId") int stringId,
            @ActionInput(name = "length") long length) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickRED)this.handler.getDevice()).truncateString(stringId, length);
        result.put("errorCode", value);
        return result;
    }

    public static Map<String, Object> brickREDTruncateString(@Nullable ThingActions actions, int stringId, long length) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDTruncateString(stringId, length);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Get String Length")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "length", type="long")
           Map<String, Object> brickREDGetStringLength(
            @ActionInput(name = "stringId") int stringId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.StringLength value = ((BrickRED)this.handler.getDevice()).getStringLength(stringId);
        result.put("errorCode", value.errorCode);
        result.put("length", value.length);
        return result;
    }

    public static Map<String, Object> brickREDGetStringLength(@Nullable ThingActions actions, int stringId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDGetStringLength(stringId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Set String Chunk")
    public @ActionOutput(name = "errorCode", type="int")
           Map<String, Object> brickREDSetStringChunk(
            @ActionInput(name = "stringId") int stringId,
            @ActionInput(name = "offset") long offset,
            @ActionInput(name = "buffer") String buffer) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickRED)this.handler.getDevice()).setStringChunk(stringId, offset, buffer);
        result.put("errorCode", value);
        return result;
    }

    public static Map<String, Object> brickREDSetStringChunk(@Nullable ThingActions actions, int stringId, long offset, String buffer) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDSetStringChunk(stringId, offset, buffer);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Get String Chunk")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "buffer", type="String")
           Map<String, Object> brickREDGetStringChunk(
            @ActionInput(name = "stringId") int stringId,
            @ActionInput(name = "offset") long offset) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.StringChunk value = ((BrickRED)this.handler.getDevice()).getStringChunk(stringId, offset);
        result.put("errorCode", value.errorCode);
        result.put("buffer", value.buffer);
        return result;
    }

    public static Map<String, Object> brickREDGetStringChunk(@Nullable ThingActions actions, int stringId, long offset) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDGetStringChunk(stringId, offset);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Allocate List")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "listId", type="int")
           Map<String, Object> brickREDAllocateList(
            @ActionInput(name = "lengthToReserve") int lengthToReserve,
            @ActionInput(name = "sessionId") int sessionId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.AllocateList value = ((BrickRED)this.handler.getDevice()).allocateList(lengthToReserve, sessionId);
        result.put("errorCode", value.errorCode);
        result.put("listId", value.listId);
        return result;
    }

    public static Map<String, Object> brickREDAllocateList(@Nullable ThingActions actions, int lengthToReserve, int sessionId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDAllocateList(lengthToReserve, sessionId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Get List Length")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "length", type="int")
           Map<String, Object> brickREDGetListLength(
            @ActionInput(name = "listId") int listId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.ListLength value = ((BrickRED)this.handler.getDevice()).getListLength(listId);
        result.put("errorCode", value.errorCode);
        result.put("length", value.length);
        return result;
    }

    public static Map<String, Object> brickREDGetListLength(@Nullable ThingActions actions, int listId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDGetListLength(listId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Get List Item")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "itemObjectId", type="int")
           @ActionOutput(name = "type", type="int")
           Map<String, Object> brickREDGetListItem(
            @ActionInput(name = "listId") int listId,
            @ActionInput(name = "index") int index,
            @ActionInput(name = "sessionId") int sessionId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.ListItem value = ((BrickRED)this.handler.getDevice()).getListItem(listId, index, sessionId);
        result.put("errorCode", value.errorCode);
        result.put("itemObjectId", value.itemObjectId);
        result.put("type", value.type);
        return result;
    }

    public static Map<String, Object> brickREDGetListItem(@Nullable ThingActions actions, int listId, int index, int sessionId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDGetListItem(listId, index, sessionId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Append To List")
    public @ActionOutput(name = "errorCode", type="int")
           Map<String, Object> brickREDAppendToList(
            @ActionInput(name = "listId") int listId,
            @ActionInput(name = "itemObjectId") int itemObjectId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickRED)this.handler.getDevice()).appendToList(listId, itemObjectId);
        result.put("errorCode", value);
        return result;
    }

    public static Map<String, Object> brickREDAppendToList(@Nullable ThingActions actions, int listId, int itemObjectId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDAppendToList(listId, itemObjectId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Remove From List")
    public @ActionOutput(name = "errorCode", type="int")
           Map<String, Object> brickREDRemoveFromList(
            @ActionInput(name = "listId") int listId,
            @ActionInput(name = "index") int index) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickRED)this.handler.getDevice()).removeFromList(listId, index);
        result.put("errorCode", value);
        return result;
    }

    public static Map<String, Object> brickREDRemoveFromList(@Nullable ThingActions actions, int listId, int index) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDRemoveFromList(listId, index);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Open File")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "fileId", type="int")
           Map<String, Object> brickREDOpenFile(
            @ActionInput(name = "nameStringId") int nameStringId,
            @ActionInput(name = "flags") long flags,
            @ActionInput(name = "permissions") int permissions,
            @ActionInput(name = "uid") long uid,
            @ActionInput(name = "gid") long gid,
            @ActionInput(name = "sessionId") int sessionId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.OpenFile value = ((BrickRED)this.handler.getDevice()).openFile(nameStringId, flags, permissions, uid, gid, sessionId);
        result.put("errorCode", value.errorCode);
        result.put("fileId", value.fileId);
        return result;
    }

    public static Map<String, Object> brickREDOpenFile(@Nullable ThingActions actions, int nameStringId, long flags, int permissions, long uid, long gid, int sessionId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDOpenFile(nameStringId, flags, permissions, uid, gid, sessionId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Create Pipe")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "fileId", type="int")
           Map<String, Object> brickREDCreatePipe(
            @ActionInput(name = "flags") long flags,
            @ActionInput(name = "length") long length,
            @ActionInput(name = "sessionId") int sessionId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.CreatePipe value = ((BrickRED)this.handler.getDevice()).createPipe(flags, length, sessionId);
        result.put("errorCode", value.errorCode);
        result.put("fileId", value.fileId);
        return result;
    }

    public static Map<String, Object> brickREDCreatePipe(@Nullable ThingActions actions, long flags, long length, int sessionId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDCreatePipe(flags, length, sessionId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Get File Info")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "type", type="int")
           @ActionOutput(name = "nameStringId", type="int")
           @ActionOutput(name = "flags", type="long")
           @ActionOutput(name = "permissions", type="int")
           @ActionOutput(name = "uid", type="long")
           @ActionOutput(name = "gid", type="long")
           @ActionOutput(name = "length", type="long")
           @ActionOutput(name = "accessTimestamp", type="long")
           @ActionOutput(name = "modificationTimestamp", type="long")
           @ActionOutput(name = "statusChangeTimestamp", type="long")
           Map<String, Object> brickREDGetFileInfo(
            @ActionInput(name = "fileId") int fileId,
            @ActionInput(name = "sessionId") int sessionId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.FileInfo value = ((BrickRED)this.handler.getDevice()).getFileInfo(fileId, sessionId);
        result.put("errorCode", value.errorCode);
        result.put("type", value.type);
        result.put("nameStringId", value.nameStringId);
        result.put("flags", value.flags);
        result.put("permissions", value.permissions);
        result.put("uid", value.uid);
        result.put("gid", value.gid);
        result.put("length", value.length);
        result.put("accessTimestamp", value.accessTimestamp);
        result.put("modificationTimestamp", value.modificationTimestamp);
        result.put("statusChangeTimestamp", value.statusChangeTimestamp);
        return result;
    }

    public static Map<String, Object> brickREDGetFileInfo(@Nullable ThingActions actions, int fileId, int sessionId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDGetFileInfo(fileId, sessionId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Read File")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "buffer", type="int[]")
           @ActionOutput(name = "lengthRead", type="int")
           Map<String, Object> brickREDReadFile(
            @ActionInput(name = "fileId") int fileId,
            @ActionInput(name = "lengthToRead") int lengthToRead) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.ReadFile value = ((BrickRED)this.handler.getDevice()).readFile(fileId, lengthToRead);
        result.put("errorCode", value.errorCode);
        result.put("buffer", value.buffer);
        result.put("lengthRead", value.lengthRead);
        return result;
    }

    public static Map<String, Object> brickREDReadFile(@Nullable ThingActions actions, int fileId, int lengthToRead) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDReadFile(fileId, lengthToRead);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Write File")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "lengthWritten", type="int")
           Map<String, Object> brickREDWriteFile(
            @ActionInput(name = "fileId") int fileId,
            @ActionInput(name = "buffer") int[] buffer,
            @ActionInput(name = "lengthToWrite") int lengthToWrite) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.WriteFile value = ((BrickRED)this.handler.getDevice()).writeFile(fileId, buffer, lengthToWrite);
        result.put("errorCode", value.errorCode);
        result.put("lengthWritten", value.lengthWritten);
        return result;
    }

    public static Map<String, Object> brickREDWriteFile(@Nullable ThingActions actions, int fileId, int[] buffer, int lengthToWrite) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDWriteFile(fileId, buffer, lengthToWrite);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Write File Unchecked")
    public void brickREDWriteFileUnchecked(
            @ActionInput(name = "fileId") int fileId,
            @ActionInput(name = "buffer") int[] buffer,
            @ActionInput(name = "lengthToWrite") int lengthToWrite) throws TinkerforgeException {
        ((BrickRED)this.handler.getDevice()).writeFileUnchecked(fileId, buffer, lengthToWrite);
    }

    public static void brickREDWriteFileUnchecked(@Nullable ThingActions actions, int fileId, int[] buffer, int lengthToWrite) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            ((BrickREDActions) actions).brickREDWriteFileUnchecked(fileId, buffer, lengthToWrite);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Set File Position")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "position", type="long")
           Map<String, Object> brickREDSetFilePosition(
            @ActionInput(name = "fileId") int fileId,
            @ActionInput(name = "offset") long offset,
            @ActionInput(name = "origin") int origin) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.SetFilePosition value = ((BrickRED)this.handler.getDevice()).setFilePosition(fileId, offset, origin);
        result.put("errorCode", value.errorCode);
        result.put("position", value.position);
        return result;
    }

    public static Map<String, Object> brickREDSetFilePosition(@Nullable ThingActions actions, int fileId, long offset, int origin) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDSetFilePosition(fileId, offset, origin);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Get File Position")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "position", type="long")
           Map<String, Object> brickREDGetFilePosition(
            @ActionInput(name = "fileId") int fileId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.FilePosition value = ((BrickRED)this.handler.getDevice()).getFilePosition(fileId);
        result.put("errorCode", value.errorCode);
        result.put("position", value.position);
        return result;
    }

    public static Map<String, Object> brickREDGetFilePosition(@Nullable ThingActions actions, int fileId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDGetFilePosition(fileId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Set File Events")
    public @ActionOutput(name = "errorCode", type="int")
           Map<String, Object> brickREDSetFileEvents(
            @ActionInput(name = "fileId") int fileId,
            @ActionInput(name = "events") int events) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickRED)this.handler.getDevice()).setFileEvents(fileId, events);
        result.put("errorCode", value);
        return result;
    }

    public static Map<String, Object> brickREDSetFileEvents(@Nullable ThingActions actions, int fileId, int events) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDSetFileEvents(fileId, events);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Get File Events")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "events", type="int")
           Map<String, Object> brickREDGetFileEvents(
            @ActionInput(name = "fileId") int fileId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.FileEvents value = ((BrickRED)this.handler.getDevice()).getFileEvents(fileId);
        result.put("errorCode", value.errorCode);
        result.put("events", value.events);
        return result;
    }

    public static Map<String, Object> brickREDGetFileEvents(@Nullable ThingActions actions, int fileId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDGetFileEvents(fileId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Open Directory")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "directoryId", type="int")
           Map<String, Object> brickREDOpenDirectory(
            @ActionInput(name = "nameStringId") int nameStringId,
            @ActionInput(name = "sessionId") int sessionId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.OpenDirectory value = ((BrickRED)this.handler.getDevice()).openDirectory(nameStringId, sessionId);
        result.put("errorCode", value.errorCode);
        result.put("directoryId", value.directoryId);
        return result;
    }

    public static Map<String, Object> brickREDOpenDirectory(@Nullable ThingActions actions, int nameStringId, int sessionId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDOpenDirectory(nameStringId, sessionId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Get Directory Name")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "nameStringId", type="int")
           Map<String, Object> brickREDGetDirectoryName(
            @ActionInput(name = "directoryId") int directoryId,
            @ActionInput(name = "sessionId") int sessionId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.DirectoryName value = ((BrickRED)this.handler.getDevice()).getDirectoryName(directoryId, sessionId);
        result.put("errorCode", value.errorCode);
        result.put("nameStringId", value.nameStringId);
        return result;
    }

    public static Map<String, Object> brickREDGetDirectoryName(@Nullable ThingActions actions, int directoryId, int sessionId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDGetDirectoryName(directoryId, sessionId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Get Next Directory Entry")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "nameStringId", type="int")
           @ActionOutput(name = "type", type="int")
           Map<String, Object> brickREDGetNextDirectoryEntry(
            @ActionInput(name = "directoryId") int directoryId,
            @ActionInput(name = "sessionId") int sessionId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.NextDirectoryEntry value = ((BrickRED)this.handler.getDevice()).getNextDirectoryEntry(directoryId, sessionId);
        result.put("errorCode", value.errorCode);
        result.put("nameStringId", value.nameStringId);
        result.put("type", value.type);
        return result;
    }

    public static Map<String, Object> brickREDGetNextDirectoryEntry(@Nullable ThingActions actions, int directoryId, int sessionId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDGetNextDirectoryEntry(directoryId, sessionId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Rewind Directory")
    public @ActionOutput(name = "errorCode", type="int")
           Map<String, Object> brickREDRewindDirectory(
            @ActionInput(name = "directoryId") int directoryId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickRED)this.handler.getDevice()).rewindDirectory(directoryId);
        result.put("errorCode", value);
        return result;
    }

    public static Map<String, Object> brickREDRewindDirectory(@Nullable ThingActions actions, int directoryId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDRewindDirectory(directoryId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Create Directory")
    public @ActionOutput(name = "errorCode", type="int")
           Map<String, Object> brickREDCreateDirectory(
            @ActionInput(name = "nameStringId") int nameStringId,
            @ActionInput(name = "flags") long flags,
            @ActionInput(name = "permissions") int permissions,
            @ActionInput(name = "uid") long uid,
            @ActionInput(name = "gid") long gid) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickRED)this.handler.getDevice()).createDirectory(nameStringId, flags, permissions, uid, gid);
        result.put("errorCode", value);
        return result;
    }

    public static Map<String, Object> brickREDCreateDirectory(@Nullable ThingActions actions, int nameStringId, long flags, int permissions, long uid, long gid) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDCreateDirectory(nameStringId, flags, permissions, uid, gid);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Get Processes")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "processesListId", type="int")
           Map<String, Object> brickREDGetProcesses(
            @ActionInput(name = "sessionId") int sessionId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.Processes value = ((BrickRED)this.handler.getDevice()).getProcesses(sessionId);
        result.put("errorCode", value.errorCode);
        result.put("processesListId", value.processesListId);
        return result;
    }

    public static Map<String, Object> brickREDGetProcesses(@Nullable ThingActions actions, int sessionId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDGetProcesses(sessionId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Spawn Process")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "processId", type="int")
           Map<String, Object> brickREDSpawnProcess(
            @ActionInput(name = "executableStringId") int executableStringId,
            @ActionInput(name = "argumentsListId") int argumentsListId,
            @ActionInput(name = "environmentListId") int environmentListId,
            @ActionInput(name = "workingDirectoryStringId") int workingDirectoryStringId,
            @ActionInput(name = "uid") long uid,
            @ActionInput(name = "gid") long gid,
            @ActionInput(name = "stdinFileId") int stdinFileId,
            @ActionInput(name = "stdoutFileId") int stdoutFileId,
            @ActionInput(name = "stderrFileId") int stderrFileId,
            @ActionInput(name = "sessionId") int sessionId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.SpawnProcess value = ((BrickRED)this.handler.getDevice()).spawnProcess(executableStringId, argumentsListId, environmentListId, workingDirectoryStringId, uid, gid, stdinFileId, stdoutFileId, stderrFileId, sessionId);
        result.put("errorCode", value.errorCode);
        result.put("processId", value.processId);
        return result;
    }

    public static Map<String, Object> brickREDSpawnProcess(@Nullable ThingActions actions, int executableStringId, int argumentsListId, int environmentListId, int workingDirectoryStringId, long uid, long gid, int stdinFileId, int stdoutFileId, int stderrFileId, int sessionId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDSpawnProcess(executableStringId, argumentsListId, environmentListId, workingDirectoryStringId, uid, gid, stdinFileId, stdoutFileId, stderrFileId, sessionId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Kill Process")
    public @ActionOutput(name = "errorCode", type="int")
           Map<String, Object> brickREDKillProcess(
            @ActionInput(name = "processId") int processId,
            @ActionInput(name = "signal") int signal) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickRED)this.handler.getDevice()).killProcess(processId, signal);
        result.put("errorCode", value);
        return result;
    }

    public static Map<String, Object> brickREDKillProcess(@Nullable ThingActions actions, int processId, int signal) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDKillProcess(processId, signal);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Get Process Command")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "executableStringId", type="int")
           @ActionOutput(name = "argumentsListId", type="int")
           @ActionOutput(name = "environmentListId", type="int")
           @ActionOutput(name = "workingDirectoryStringId", type="int")
           Map<String, Object> brickREDGetProcessCommand(
            @ActionInput(name = "processId") int processId,
            @ActionInput(name = "sessionId") int sessionId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.ProcessCommand value = ((BrickRED)this.handler.getDevice()).getProcessCommand(processId, sessionId);
        result.put("errorCode", value.errorCode);
        result.put("executableStringId", value.executableStringId);
        result.put("argumentsListId", value.argumentsListId);
        result.put("environmentListId", value.environmentListId);
        result.put("workingDirectoryStringId", value.workingDirectoryStringId);
        return result;
    }

    public static Map<String, Object> brickREDGetProcessCommand(@Nullable ThingActions actions, int processId, int sessionId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDGetProcessCommand(processId, sessionId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Get Process Identity")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "pid", type="long")
           @ActionOutput(name = "uid", type="long")
           @ActionOutput(name = "gid", type="long")
           Map<String, Object> brickREDGetProcessIdentity(
            @ActionInput(name = "processId") int processId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.ProcessIdentity value = ((BrickRED)this.handler.getDevice()).getProcessIdentity(processId);
        result.put("errorCode", value.errorCode);
        result.put("pid", value.pid);
        result.put("uid", value.uid);
        result.put("gid", value.gid);
        return result;
    }

    public static Map<String, Object> brickREDGetProcessIdentity(@Nullable ThingActions actions, int processId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDGetProcessIdentity(processId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Get Process Stdio")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "stdinFileId", type="int")
           @ActionOutput(name = "stdoutFileId", type="int")
           @ActionOutput(name = "stderrFileId", type="int")
           Map<String, Object> brickREDGetProcessStdio(
            @ActionInput(name = "processId") int processId,
            @ActionInput(name = "sessionId") int sessionId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.ProcessStdio value = ((BrickRED)this.handler.getDevice()).getProcessStdio(processId, sessionId);
        result.put("errorCode", value.errorCode);
        result.put("stdinFileId", value.stdinFileId);
        result.put("stdoutFileId", value.stdoutFileId);
        result.put("stderrFileId", value.stderrFileId);
        return result;
    }

    public static Map<String, Object> brickREDGetProcessStdio(@Nullable ThingActions actions, int processId, int sessionId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDGetProcessStdio(processId, sessionId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Get Process State")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "state", type="int")
           @ActionOutput(name = "timestamp", type="long")
           @ActionOutput(name = "exitCode", type="int")
           Map<String, Object> brickREDGetProcessState(
            @ActionInput(name = "processId") int processId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.ProcessState value = ((BrickRED)this.handler.getDevice()).getProcessState(processId);
        result.put("errorCode", value.errorCode);
        result.put("state", value.state);
        result.put("timestamp", value.timestamp);
        result.put("exitCode", value.exitCode);
        return result;
    }

    public static Map<String, Object> brickREDGetProcessState(@Nullable ThingActions actions, int processId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDGetProcessState(processId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Get Programs")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "programsListId", type="int")
           Map<String, Object> brickREDGetPrograms(
            @ActionInput(name = "sessionId") int sessionId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.Programs value = ((BrickRED)this.handler.getDevice()).getPrograms(sessionId);
        result.put("errorCode", value.errorCode);
        result.put("programsListId", value.programsListId);
        return result;
    }

    public static Map<String, Object> brickREDGetPrograms(@Nullable ThingActions actions, int sessionId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDGetPrograms(sessionId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Define Program")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "programId", type="int")
           Map<String, Object> brickREDDefineProgram(
            @ActionInput(name = "identifierStringId") int identifierStringId,
            @ActionInput(name = "sessionId") int sessionId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.DefineProgram value = ((BrickRED)this.handler.getDevice()).defineProgram(identifierStringId, sessionId);
        result.put("errorCode", value.errorCode);
        result.put("programId", value.programId);
        return result;
    }

    public static Map<String, Object> brickREDDefineProgram(@Nullable ThingActions actions, int identifierStringId, int sessionId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDDefineProgram(identifierStringId, sessionId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Purge Program")
    public @ActionOutput(name = "errorCode", type="int")
           Map<String, Object> brickREDPurgeProgram(
            @ActionInput(name = "programId") int programId,
            @ActionInput(name = "cookie") long cookie) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickRED)this.handler.getDevice()).purgeProgram(programId, cookie);
        result.put("errorCode", value);
        return result;
    }

    public static Map<String, Object> brickREDPurgeProgram(@Nullable ThingActions actions, int programId, long cookie) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDPurgeProgram(programId, cookie);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Get Program Identifier")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "identifierStringId", type="int")
           Map<String, Object> brickREDGetProgramIdentifier(
            @ActionInput(name = "programId") int programId,
            @ActionInput(name = "sessionId") int sessionId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.ProgramIdentifier value = ((BrickRED)this.handler.getDevice()).getProgramIdentifier(programId, sessionId);
        result.put("errorCode", value.errorCode);
        result.put("identifierStringId", value.identifierStringId);
        return result;
    }

    public static Map<String, Object> brickREDGetProgramIdentifier(@Nullable ThingActions actions, int programId, int sessionId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDGetProgramIdentifier(programId, sessionId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Get Program Root Directory")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "rootDirectoryStringId", type="int")
           Map<String, Object> brickREDGetProgramRootDirectory(
            @ActionInput(name = "programId") int programId,
            @ActionInput(name = "sessionId") int sessionId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.ProgramRootDirectory value = ((BrickRED)this.handler.getDevice()).getProgramRootDirectory(programId, sessionId);
        result.put("errorCode", value.errorCode);
        result.put("rootDirectoryStringId", value.rootDirectoryStringId);
        return result;
    }

    public static Map<String, Object> brickREDGetProgramRootDirectory(@Nullable ThingActions actions, int programId, int sessionId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDGetProgramRootDirectory(programId, sessionId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Set Program Command")
    public @ActionOutput(name = "errorCode", type="int")
           Map<String, Object> brickREDSetProgramCommand(
            @ActionInput(name = "programId") int programId,
            @ActionInput(name = "executableStringId") int executableStringId,
            @ActionInput(name = "argumentsListId") int argumentsListId,
            @ActionInput(name = "environmentListId") int environmentListId,
            @ActionInput(name = "workingDirectoryStringId") int workingDirectoryStringId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickRED)this.handler.getDevice()).setProgramCommand(programId, executableStringId, argumentsListId, environmentListId, workingDirectoryStringId);
        result.put("errorCode", value);
        return result;
    }

    public static Map<String, Object> brickREDSetProgramCommand(@Nullable ThingActions actions, int programId, int executableStringId, int argumentsListId, int environmentListId, int workingDirectoryStringId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDSetProgramCommand(programId, executableStringId, argumentsListId, environmentListId, workingDirectoryStringId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Get Program Command")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "executableStringId", type="int")
           @ActionOutput(name = "argumentsListId", type="int")
           @ActionOutput(name = "environmentListId", type="int")
           @ActionOutput(name = "workingDirectoryStringId", type="int")
           Map<String, Object> brickREDGetProgramCommand(
            @ActionInput(name = "programId") int programId,
            @ActionInput(name = "sessionId") int sessionId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.ProgramCommand value = ((BrickRED)this.handler.getDevice()).getProgramCommand(programId, sessionId);
        result.put("errorCode", value.errorCode);
        result.put("executableStringId", value.executableStringId);
        result.put("argumentsListId", value.argumentsListId);
        result.put("environmentListId", value.environmentListId);
        result.put("workingDirectoryStringId", value.workingDirectoryStringId);
        return result;
    }

    public static Map<String, Object> brickREDGetProgramCommand(@Nullable ThingActions actions, int programId, int sessionId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDGetProgramCommand(programId, sessionId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Set Program Stdio Redirection")
    public @ActionOutput(name = "errorCode", type="int")
           Map<String, Object> brickREDSetProgramStdioRedirection(
            @ActionInput(name = "programId") int programId,
            @ActionInput(name = "stdinRedirection") int stdinRedirection,
            @ActionInput(name = "stdinFileNameStringId") int stdinFileNameStringId,
            @ActionInput(name = "stdoutRedirection") int stdoutRedirection,
            @ActionInput(name = "stdoutFileNameStringId") int stdoutFileNameStringId,
            @ActionInput(name = "stderrRedirection") int stderrRedirection,
            @ActionInput(name = "stderrFileNameStringId") int stderrFileNameStringId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickRED)this.handler.getDevice()).setProgramStdioRedirection(programId, stdinRedirection, stdinFileNameStringId, stdoutRedirection, stdoutFileNameStringId, stderrRedirection, stderrFileNameStringId);
        result.put("errorCode", value);
        return result;
    }

    public static Map<String, Object> brickREDSetProgramStdioRedirection(@Nullable ThingActions actions, int programId, int stdinRedirection, int stdinFileNameStringId, int stdoutRedirection, int stdoutFileNameStringId, int stderrRedirection, int stderrFileNameStringId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDSetProgramStdioRedirection(programId, stdinRedirection, stdinFileNameStringId, stdoutRedirection, stdoutFileNameStringId, stderrRedirection, stderrFileNameStringId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Get Program Stdio Redirection")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "stdinRedirection", type="int")
           @ActionOutput(name = "stdinFileNameStringId", type="int")
           @ActionOutput(name = "stdoutRedirection", type="int")
           @ActionOutput(name = "stdoutFileNameStringId", type="int")
           @ActionOutput(name = "stderrRedirection", type="int")
           @ActionOutput(name = "stderrFileNameStringId", type="int")
           Map<String, Object> brickREDGetProgramStdioRedirection(
            @ActionInput(name = "programId") int programId,
            @ActionInput(name = "sessionId") int sessionId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.ProgramStdioRedirection value = ((BrickRED)this.handler.getDevice()).getProgramStdioRedirection(programId, sessionId);
        result.put("errorCode", value.errorCode);
        result.put("stdinRedirection", value.stdinRedirection);
        result.put("stdinFileNameStringId", value.stdinFileNameStringId);
        result.put("stdoutRedirection", value.stdoutRedirection);
        result.put("stdoutFileNameStringId", value.stdoutFileNameStringId);
        result.put("stderrRedirection", value.stderrRedirection);
        result.put("stderrFileNameStringId", value.stderrFileNameStringId);
        return result;
    }

    public static Map<String, Object> brickREDGetProgramStdioRedirection(@Nullable ThingActions actions, int programId, int sessionId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDGetProgramStdioRedirection(programId, sessionId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Set Program Schedule")
    public @ActionOutput(name = "errorCode", type="int")
           Map<String, Object> brickREDSetProgramSchedule(
            @ActionInput(name = "programId") int programId,
            @ActionInput(name = "startMode") int startMode,
            @ActionInput(name = "continueAfterError") boolean continueAfterError,
            @ActionInput(name = "startInterval") long startInterval,
            @ActionInput(name = "startFieldsStringId") int startFieldsStringId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickRED)this.handler.getDevice()).setProgramSchedule(programId, startMode, continueAfterError, startInterval, startFieldsStringId);
        result.put("errorCode", value);
        return result;
    }

    public static Map<String, Object> brickREDSetProgramSchedule(@Nullable ThingActions actions, int programId, int startMode, boolean continueAfterError, long startInterval, int startFieldsStringId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDSetProgramSchedule(programId, startMode, continueAfterError, startInterval, startFieldsStringId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Get Program Schedule")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "startMode", type="int")
           @ActionOutput(name = "continueAfterError", type="boolean")
           @ActionOutput(name = "startInterval", type="long")
           @ActionOutput(name = "startFieldsStringId", type="int")
           Map<String, Object> brickREDGetProgramSchedule(
            @ActionInput(name = "programId") int programId,
            @ActionInput(name = "sessionId") int sessionId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.ProgramSchedule value = ((BrickRED)this.handler.getDevice()).getProgramSchedule(programId, sessionId);
        result.put("errorCode", value.errorCode);
        result.put("startMode", value.startMode);
        result.put("continueAfterError", value.continueAfterError);
        result.put("startInterval", value.startInterval);
        result.put("startFieldsStringId", value.startFieldsStringId);
        return result;
    }

    public static Map<String, Object> brickREDGetProgramSchedule(@Nullable ThingActions actions, int programId, int sessionId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDGetProgramSchedule(programId, sessionId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Get Program Scheduler State")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "state", type="int")
           @ActionOutput(name = "timestamp", type="long")
           @ActionOutput(name = "messageStringId", type="int")
           Map<String, Object> brickREDGetProgramSchedulerState(
            @ActionInput(name = "programId") int programId,
            @ActionInput(name = "sessionId") int sessionId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.ProgramSchedulerState value = ((BrickRED)this.handler.getDevice()).getProgramSchedulerState(programId, sessionId);
        result.put("errorCode", value.errorCode);
        result.put("state", value.state);
        result.put("timestamp", value.timestamp);
        result.put("messageStringId", value.messageStringId);
        return result;
    }

    public static Map<String, Object> brickREDGetProgramSchedulerState(@Nullable ThingActions actions, int programId, int sessionId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDGetProgramSchedulerState(programId, sessionId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Continue Program Schedule")
    public @ActionOutput(name = "errorCode", type="int")
           Map<String, Object> brickREDContinueProgramSchedule(
            @ActionInput(name = "programId") int programId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickRED)this.handler.getDevice()).continueProgramSchedule(programId);
        result.put("errorCode", value);
        return result;
    }

    public static Map<String, Object> brickREDContinueProgramSchedule(@Nullable ThingActions actions, int programId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDContinueProgramSchedule(programId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Start Program")
    public @ActionOutput(name = "errorCode", type="int")
           Map<String, Object> brickREDStartProgram(
            @ActionInput(name = "programId") int programId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickRED)this.handler.getDevice()).startProgram(programId);
        result.put("errorCode", value);
        return result;
    }

    public static Map<String, Object> brickREDStartProgram(@Nullable ThingActions actions, int programId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDStartProgram(programId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Get Last Spawned Program Process")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "processId", type="int")
           @ActionOutput(name = "timestamp", type="long")
           Map<String, Object> brickREDGetLastSpawnedProgramProcess(
            @ActionInput(name = "programId") int programId,
            @ActionInput(name = "sessionId") int sessionId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.LastSpawnedProgramProcess value = ((BrickRED)this.handler.getDevice()).getLastSpawnedProgramProcess(programId, sessionId);
        result.put("errorCode", value.errorCode);
        result.put("processId", value.processId);
        result.put("timestamp", value.timestamp);
        return result;
    }

    public static Map<String, Object> brickREDGetLastSpawnedProgramProcess(@Nullable ThingActions actions, int programId, int sessionId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDGetLastSpawnedProgramProcess(programId, sessionId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Get Custom Program Option Names")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "namesListId", type="int")
           Map<String, Object> brickREDGetCustomProgramOptionNames(
            @ActionInput(name = "programId") int programId,
            @ActionInput(name = "sessionId") int sessionId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.CustomProgramOptionNames value = ((BrickRED)this.handler.getDevice()).getCustomProgramOptionNames(programId, sessionId);
        result.put("errorCode", value.errorCode);
        result.put("namesListId", value.namesListId);
        return result;
    }

    public static Map<String, Object> brickREDGetCustomProgramOptionNames(@Nullable ThingActions actions, int programId, int sessionId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDGetCustomProgramOptionNames(programId, sessionId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Set Custom Program Option Value")
    public @ActionOutput(name = "errorCode", type="int")
           Map<String, Object> brickREDSetCustomProgramOptionValue(
            @ActionInput(name = "programId") int programId,
            @ActionInput(name = "nameStringId") int nameStringId,
            @ActionInput(name = "valueStringId") int valueStringId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickRED)this.handler.getDevice()).setCustomProgramOptionValue(programId, nameStringId, valueStringId);
        result.put("errorCode", value);
        return result;
    }

    public static Map<String, Object> brickREDSetCustomProgramOptionValue(@Nullable ThingActions actions, int programId, int nameStringId, int valueStringId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDSetCustomProgramOptionValue(programId, nameStringId, valueStringId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Get Custom Program Option Value")
    public @ActionOutput(name = "errorCode", type="int")
           @ActionOutput(name = "valueStringId", type="int")
           Map<String, Object> brickREDGetCustomProgramOptionValue(
            @ActionInput(name = "programId") int programId,
            @ActionInput(name = "nameStringId") int nameStringId,
            @ActionInput(name = "sessionId") int sessionId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.CustomProgramOptionValue value = ((BrickRED)this.handler.getDevice()).getCustomProgramOptionValue(programId, nameStringId, sessionId);
        result.put("errorCode", value.errorCode);
        result.put("valueStringId", value.valueStringId);
        return result;
    }

    public static Map<String, Object> brickREDGetCustomProgramOptionValue(@Nullable ThingActions actions, int programId, int nameStringId, int sessionId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDGetCustomProgramOptionValue(programId, nameStringId, sessionId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Remove Custom Program Option")
    public @ActionOutput(name = "errorCode", type="int")
           Map<String, Object> brickREDRemoveCustomProgramOption(
            @ActionInput(name = "programId") int programId,
            @ActionInput(name = "nameStringId") int nameStringId) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickRED)this.handler.getDevice()).removeCustomProgramOption(programId, nameStringId);
        result.put("errorCode", value);
        return result;
    }

    public static Map<String, Object> brickREDRemoveCustomProgramOption(@Nullable ThingActions actions, int programId, int nameStringId) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDRemoveCustomProgramOption(programId, nameStringId);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickREDGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickRED.Identity value = ((BrickRED)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickREDGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickREDActions) {
            return ((BrickREDActions) actions).brickREDGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickREDActions class.");
        }
    }
}
