package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickStepperActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Set Max Velocity")
    public void brickStepperSetMaxVelocity(
            @ActionInput(name = "velocity") int velocity) throws TinkerforgeException {
        ((BrickStepper)this.handler.getDevice()).setMaxVelocity(velocity);
    }

    public static void brickStepperSetMaxVelocity(@Nullable ThingActions actions, int velocity) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            ((BrickStepperActions) actions).brickStepperSetMaxVelocity(velocity);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Get Max Velocity")
    public @ActionOutput(name = "velocity", type="int")
           Map<String, Object> brickStepperGetMaxVelocity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickStepper)this.handler.getDevice()).getMaxVelocity();
        result.put("velocity", value);
        return result;
    }

    public static Map<String, Object> brickStepperGetMaxVelocity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            return ((BrickStepperActions) actions).brickStepperGetMaxVelocity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Get Current Velocity")
    public @ActionOutput(name = "velocity", type="int")
           Map<String, Object> brickStepperGetCurrentVelocity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickStepper)this.handler.getDevice()).getCurrentVelocity();
        result.put("velocity", value);
        return result;
    }

    public static Map<String, Object> brickStepperGetCurrentVelocity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            return ((BrickStepperActions) actions).brickStepperGetCurrentVelocity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Set Speed Ramping")
    public void brickStepperSetSpeedRamping(
            @ActionInput(name = "acceleration") int acceleration,
            @ActionInput(name = "deacceleration") int deacceleration) throws TinkerforgeException {
        ((BrickStepper)this.handler.getDevice()).setSpeedRamping(acceleration, deacceleration);
    }

    public static void brickStepperSetSpeedRamping(@Nullable ThingActions actions, int acceleration, int deacceleration) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            ((BrickStepperActions) actions).brickStepperSetSpeedRamping(acceleration, deacceleration);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Get Speed Ramping")
    public @ActionOutput(name = "acceleration", type="int")
           @ActionOutput(name = "deacceleration", type="int")
           Map<String, Object> brickStepperGetSpeedRamping(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickStepper.SpeedRamping value = ((BrickStepper)this.handler.getDevice()).getSpeedRamping();
        result.put("acceleration", value.acceleration);
        result.put("deacceleration", value.deacceleration);
        return result;
    }

    public static Map<String, Object> brickStepperGetSpeedRamping(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            return ((BrickStepperActions) actions).brickStepperGetSpeedRamping();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Full Brake")
    public void brickStepperFullBrake(
            ) throws TinkerforgeException {
        ((BrickStepper)this.handler.getDevice()).fullBrake();
    }

    public static void brickStepperFullBrake(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            ((BrickStepperActions) actions).brickStepperFullBrake();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Set Steps")
    public void brickStepperSetSteps(
            @ActionInput(name = "steps") int steps) throws TinkerforgeException {
        ((BrickStepper)this.handler.getDevice()).setSteps(steps);
    }

    public static void brickStepperSetSteps(@Nullable ThingActions actions, int steps) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            ((BrickStepperActions) actions).brickStepperSetSteps(steps);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Get Steps")
    public @ActionOutput(name = "steps", type="int")
           Map<String, Object> brickStepperGetSteps(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickStepper)this.handler.getDevice()).getSteps();
        result.put("steps", value);
        return result;
    }

    public static Map<String, Object> brickStepperGetSteps(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            return ((BrickStepperActions) actions).brickStepperGetSteps();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Get Remaining Steps")
    public @ActionOutput(name = "steps", type="int")
           Map<String, Object> brickStepperGetRemainingSteps(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickStepper)this.handler.getDevice()).getRemainingSteps();
        result.put("steps", value);
        return result;
    }

    public static Map<String, Object> brickStepperGetRemainingSteps(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            return ((BrickStepperActions) actions).brickStepperGetRemainingSteps();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Drive Forward")
    public void brickStepperDriveForward(
            ) throws TinkerforgeException {
        ((BrickStepper)this.handler.getDevice()).driveForward();
    }

    public static void brickStepperDriveForward(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            ((BrickStepperActions) actions).brickStepperDriveForward();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Drive Backward")
    public void brickStepperDriveBackward(
            ) throws TinkerforgeException {
        ((BrickStepper)this.handler.getDevice()).driveBackward();
    }

    public static void brickStepperDriveBackward(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            ((BrickStepperActions) actions).brickStepperDriveBackward();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Stop")
    public void brickStepperStop(
            ) throws TinkerforgeException {
        ((BrickStepper)this.handler.getDevice()).stop();
    }

    public static void brickStepperStop(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            ((BrickStepperActions) actions).brickStepperStop();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Set Motor Current")
    public void brickStepperSetMotorCurrent(
            @ActionInput(name = "current") int current) throws TinkerforgeException {
        ((BrickStepper)this.handler.getDevice()).setMotorCurrent(current);
    }

    public static void brickStepperSetMotorCurrent(@Nullable ThingActions actions, int current) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            ((BrickStepperActions) actions).brickStepperSetMotorCurrent(current);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Get Motor Current")
    public @ActionOutput(name = "current", type="int")
           Map<String, Object> brickStepperGetMotorCurrent(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickStepper)this.handler.getDevice()).getMotorCurrent();
        result.put("current", value);
        return result;
    }

    public static Map<String, Object> brickStepperGetMotorCurrent(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            return ((BrickStepperActions) actions).brickStepperGetMotorCurrent();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Enable")
    public void brickStepperEnable(
            ) throws TinkerforgeException {
        ((BrickStepper)this.handler.getDevice()).enable();
    }

    public static void brickStepperEnable(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            ((BrickStepperActions) actions).brickStepperEnable();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Disable")
    public void brickStepperDisable(
            ) throws TinkerforgeException {
        ((BrickStepper)this.handler.getDevice()).disable();
    }

    public static void brickStepperDisable(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            ((BrickStepperActions) actions).brickStepperDisable();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Is Enabled")
    public @ActionOutput(name = "enabled", type="boolean")
           Map<String, Object> brickStepperIsEnabled(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickStepper)this.handler.getDevice()).isEnabled();
        result.put("enabled", value);
        return result;
    }

    public static Map<String, Object> brickStepperIsEnabled(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            return ((BrickStepperActions) actions).brickStepperIsEnabled();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Set Current Position")
    public void brickStepperSetCurrentPosition(
            @ActionInput(name = "position") int position) throws TinkerforgeException {
        ((BrickStepper)this.handler.getDevice()).setCurrentPosition(position);
    }

    public static void brickStepperSetCurrentPosition(@Nullable ThingActions actions, int position) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            ((BrickStepperActions) actions).brickStepperSetCurrentPosition(position);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Get Current Position")
    public @ActionOutput(name = "position", type="int")
           Map<String, Object> brickStepperGetCurrentPosition(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickStepper)this.handler.getDevice()).getCurrentPosition();
        result.put("position", value);
        return result;
    }

    public static Map<String, Object> brickStepperGetCurrentPosition(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            return ((BrickStepperActions) actions).brickStepperGetCurrentPosition();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Set Target Position")
    public void brickStepperSetTargetPosition(
            @ActionInput(name = "position") int position) throws TinkerforgeException {
        ((BrickStepper)this.handler.getDevice()).setTargetPosition(position);
    }

    public static void brickStepperSetTargetPosition(@Nullable ThingActions actions, int position) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            ((BrickStepperActions) actions).brickStepperSetTargetPosition(position);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Get Target Position")
    public @ActionOutput(name = "position", type="int")
           Map<String, Object> brickStepperGetTargetPosition(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickStepper)this.handler.getDevice()).getTargetPosition();
        result.put("position", value);
        return result;
    }

    public static Map<String, Object> brickStepperGetTargetPosition(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            return ((BrickStepperActions) actions).brickStepperGetTargetPosition();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Set Step Mode")
    public void brickStepperSetStepMode(
            @ActionInput(name = "mode") int mode) throws TinkerforgeException {
        ((BrickStepper)this.handler.getDevice()).setStepMode(mode);
    }

    public static void brickStepperSetStepMode(@Nullable ThingActions actions, int mode) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            ((BrickStepperActions) actions).brickStepperSetStepMode(mode);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Get Step Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickStepperGetStepMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickStepper)this.handler.getDevice()).getStepMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickStepperGetStepMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            return ((BrickStepperActions) actions).brickStepperGetStepMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Get Stack Input Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> brickStepperGetStackInputVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickStepper)this.handler.getDevice()).getStackInputVoltage();
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickStepperGetStackInputVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            return ((BrickStepperActions) actions).brickStepperGetStackInputVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Get External Input Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> brickStepperGetExternalInputVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickStepper)this.handler.getDevice()).getExternalInputVoltage();
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickStepperGetExternalInputVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            return ((BrickStepperActions) actions).brickStepperGetExternalInputVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Get Current Consumption")
    public @ActionOutput(name = "current", type="int")
           Map<String, Object> brickStepperGetCurrentConsumption(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickStepper)this.handler.getDevice()).getCurrentConsumption();
        result.put("current", value);
        return result;
    }

    public static Map<String, Object> brickStepperGetCurrentConsumption(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            return ((BrickStepperActions) actions).brickStepperGetCurrentConsumption();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Set Decay")
    public void brickStepperSetDecay(
            @ActionInput(name = "decay") int decay) throws TinkerforgeException {
        ((BrickStepper)this.handler.getDevice()).setDecay(decay);
    }

    public static void brickStepperSetDecay(@Nullable ThingActions actions, int decay) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            ((BrickStepperActions) actions).brickStepperSetDecay(decay);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Get Decay")
    public @ActionOutput(name = "decay", type="int")
           Map<String, Object> brickStepperGetDecay(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickStepper)this.handler.getDevice()).getDecay();
        result.put("decay", value);
        return result;
    }

    public static Map<String, Object> brickStepperGetDecay(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            return ((BrickStepperActions) actions).brickStepperGetDecay();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Set Sync Rect")
    public void brickStepperSetSyncRect(
            @ActionInput(name = "syncRect") boolean syncRect) throws TinkerforgeException {
        ((BrickStepper)this.handler.getDevice()).setSyncRect(syncRect);
    }

    public static void brickStepperSetSyncRect(@Nullable ThingActions actions, boolean syncRect) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            ((BrickStepperActions) actions).brickStepperSetSyncRect(syncRect);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Is Sync Rect")
    public @ActionOutput(name = "syncRect", type="boolean")
           Map<String, Object> brickStepperIsSyncRect(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickStepper)this.handler.getDevice()).isSyncRect();
        result.put("syncRect", value);
        return result;
    }

    public static Map<String, Object> brickStepperIsSyncRect(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            return ((BrickStepperActions) actions).brickStepperIsSyncRect();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Set Time Base")
    public void brickStepperSetTimeBase(
            @ActionInput(name = "timeBase") long timeBase) throws TinkerforgeException {
        ((BrickStepper)this.handler.getDevice()).setTimeBase(timeBase);
    }

    public static void brickStepperSetTimeBase(@Nullable ThingActions actions, long timeBase) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            ((BrickStepperActions) actions).brickStepperSetTimeBase(timeBase);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Get Time Base")
    public @ActionOutput(name = "timeBase", type="long")
           Map<String, Object> brickStepperGetTimeBase(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickStepper)this.handler.getDevice()).getTimeBase();
        result.put("timeBase", value);
        return result;
    }

    public static Map<String, Object> brickStepperGetTimeBase(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            return ((BrickStepperActions) actions).brickStepperGetTimeBase();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Get All Data")
    public @ActionOutput(name = "currentVelocity", type="int")
           @ActionOutput(name = "currentPosition", type="int")
           @ActionOutput(name = "remainingSteps", type="int")
           @ActionOutput(name = "stackVoltage", type="int")
           @ActionOutput(name = "externalVoltage", type="int")
           @ActionOutput(name = "currentConsumption", type="int")
           Map<String, Object> brickStepperGetAllData(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickStepper.AllData value = ((BrickStepper)this.handler.getDevice()).getAllData();
        result.put("currentVelocity", value.currentVelocity);
        result.put("currentPosition", value.currentPosition);
        result.put("remainingSteps", value.remainingSteps);
        result.put("stackVoltage", value.stackVoltage);
        result.put("externalVoltage", value.externalVoltage);
        result.put("currentConsumption", value.currentConsumption);
        return result;
    }

    public static Map<String, Object> brickStepperGetAllData(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            return ((BrickStepperActions) actions).brickStepperGetAllData();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Get All Data Period")
    public @ActionOutput(name = "period", type="long")
           Map<String, Object> brickStepperGetAllDataPeriod(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickStepper)this.handler.getDevice()).getAllDataPeriod();
        result.put("period", value);
        return result;
    }

    public static Map<String, Object> brickStepperGetAllDataPeriod(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            return ((BrickStepperActions) actions).brickStepperGetAllDataPeriod();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickStepperGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickStepper.Identity value = ((BrickStepper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickStepperGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            return ((BrickStepperActions) actions).brickStepperGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Is Status LED Enabled")
    public @ActionOutput(name = "enabled", type="boolean")
           Map<String, Object> brickStepperIsStatusLEDEnabled(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickStepper)this.handler.getDevice()).isStatusLEDEnabled();
        result.put("enabled", value);
        return result;
    }

    public static Map<String, Object> brickStepperIsStatusLEDEnabled(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            return ((BrickStepperActions) actions).brickStepperIsStatusLEDEnabled();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickStepperGetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickStepper)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickStepperGetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            return ((BrickStepperActions) actions).brickStepperGetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Get Send Timeout Count")
    public @ActionOutput(name = "timeoutCount", type="long")
           Map<String, Object> brickStepperGetSendTimeoutCount(
            @ActionInput(name = "communicationMethod") int communicationMethod) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickStepper)this.handler.getDevice()).getSendTimeoutCount(communicationMethod);
        result.put("timeoutCount", value);
        return result;
    }

    public static Map<String, Object> brickStepperGetSendTimeoutCount(@Nullable ThingActions actions, int communicationMethod) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            return ((BrickStepperActions) actions).brickStepperGetSendTimeoutCount(communicationMethod);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Baudrate Config")
    public @ActionOutput(name = "enableDynamicBaudrate", type="boolean")
           @ActionOutput(name = "minimumDynamicBaudrate", type="long")
           Map<String, Object> brickStepperGetSPITFPBaudrateConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickStepper.SPITFPBaudrateConfig value = ((BrickStepper)this.handler.getDevice()).getSPITFPBaudrateConfig();
        result.put("enableDynamicBaudrate", value.enableDynamicBaudrate);
        result.put("minimumDynamicBaudrate", value.minimumDynamicBaudrate);
        return result;
    }

    public static Map<String, Object> brickStepperGetSPITFPBaudrateConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            return ((BrickStepperActions) actions).brickStepperGetSPITFPBaudrateConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountACKChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickStepperGetSPITFPErrorCount(
            @ActionInput(name = "brickletPort") char brickletPort) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickStepper.SPITFPErrorCount value = ((BrickStepper)this.handler.getDevice()).getSPITFPErrorCount(brickletPort);
        result.put("errorCountACKChecksum", value.errorCountACKChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickStepperGetSPITFPErrorCount(@Nullable ThingActions actions, char brickletPort) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            return ((BrickStepperActions) actions).brickStepperGetSPITFPErrorCount(brickletPort);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Baudrate")
    public @ActionOutput(name = "baudrate", type="long")
           Map<String, Object> brickStepperGetSPITFPBaudrate(
            @ActionInput(name = "brickletPort") char brickletPort) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickStepper)this.handler.getDevice()).getSPITFPBaudrate(brickletPort);
        result.put("baudrate", value);
        return result;
    }

    public static Map<String, Object> brickStepperGetSPITFPBaudrate(@Nullable ThingActions actions, char brickletPort) throws TinkerforgeException {
        if (actions instanceof BrickStepperActions) {
            return ((BrickStepperActions) actions).brickStepperGetSPITFPBaudrate(brickletPort);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickStepperActions class.");
        }
    }
}
