package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletBarometerActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Air Pressure")
    public @ActionOutput(name = "airPressure", type="int")
           Map<String, Object> brickletBarometerGetAirPressure(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletBarometer)this.handler.getDevice()).getAirPressure();
        result.put("airPressure", value);
        return result;
    }

    public static Map<String, Object> brickletBarometerGetAirPressure(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletBarometerActions) {
            return ((BrickletBarometerActions) actions).brickletBarometerGetAirPressure();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletBarometerActions class.");
        }
    }

    @RuleAction(label = "Get Altitude")
    public @ActionOutput(name = "altitude", type="int")
           Map<String, Object> brickletBarometerGetAltitude(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletBarometer)this.handler.getDevice()).getAltitude();
        result.put("altitude", value);
        return result;
    }

    public static Map<String, Object> brickletBarometerGetAltitude(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletBarometerActions) {
            return ((BrickletBarometerActions) actions).brickletBarometerGetAltitude();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletBarometerActions class.");
        }
    }

    @RuleAction(label = "Get Reference Air Pressure")
    public @ActionOutput(name = "airPressure", type="int")
           Map<String, Object> brickletBarometerGetReferenceAirPressure(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletBarometer)this.handler.getDevice()).getReferenceAirPressure();
        result.put("airPressure", value);
        return result;
    }

    public static Map<String, Object> brickletBarometerGetReferenceAirPressure(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletBarometerActions) {
            return ((BrickletBarometerActions) actions).brickletBarometerGetReferenceAirPressure();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletBarometerActions class.");
        }
    }

    @RuleAction(label = "Get Averaging")
    public @ActionOutput(name = "movingAveragePressure", type="int")
           @ActionOutput(name = "averagePressure", type="int")
           @ActionOutput(name = "averageTemperature", type="int")
           Map<String, Object> brickletBarometerGetAveraging(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletBarometer.Averaging value = ((BrickletBarometer)this.handler.getDevice()).getAveraging();
        result.put("movingAveragePressure", value.movingAveragePressure);
        result.put("averagePressure", value.averagePressure);
        result.put("averageTemperature", value.averageTemperature);
        return result;
    }

    public static Map<String, Object> brickletBarometerGetAveraging(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletBarometerActions) {
            return ((BrickletBarometerActions) actions).brickletBarometerGetAveraging();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletBarometerActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletBarometerGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletBarometer.Identity value = ((BrickletBarometer)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletBarometerGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletBarometerActions) {
            return ((BrickletBarometerActions) actions).brickletBarometerGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletBarometerActions class.");
        }
    }
}
