package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletLCD16x2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Write Line")
    public void brickletLCD16x2WriteLine(
            @ActionInput(name = "line") int line,
            @ActionInput(name = "position") int position,
            @ActionInput(name = "text") String text) throws TinkerforgeException {
        ((BrickletLCD16x2)this.handler.getDevice()).writeLine(line, position, text);
    }

    public static void brickletLCD16x2WriteLine(@Nullable ThingActions actions, int line, int position, String text) throws TinkerforgeException {
        if (actions instanceof BrickletLCD16x2Actions) {
            ((BrickletLCD16x2Actions) actions).brickletLCD16x2WriteLine(line, position, text);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD16x2Actions class.");
        }
    }

    @RuleAction(label = "Clear Display")
    public void brickletLCD16x2ClearDisplay(
            ) throws TinkerforgeException {
        ((BrickletLCD16x2)this.handler.getDevice()).clearDisplay();
    }

    public static void brickletLCD16x2ClearDisplay(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLCD16x2Actions) {
            ((BrickletLCD16x2Actions) actions).brickletLCD16x2ClearDisplay();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD16x2Actions class.");
        }
    }

    @RuleAction(label = "Is Backlight On")
    public @ActionOutput(name = "backlight", type="boolean")
           Map<String, Object> brickletLCD16x2IsBacklightOn(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickletLCD16x2)this.handler.getDevice()).isBacklightOn();
        result.put("backlight", value);
        return result;
    }

    public static Map<String, Object> brickletLCD16x2IsBacklightOn(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLCD16x2Actions) {
            return ((BrickletLCD16x2Actions) actions).brickletLCD16x2IsBacklightOn();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD16x2Actions class.");
        }
    }

    @RuleAction(label = "Get Config")
    public @ActionOutput(name = "cursor", type="boolean")
           @ActionOutput(name = "blinking", type="boolean")
           Map<String, Object> brickletLCD16x2GetConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletLCD16x2.Config value = ((BrickletLCD16x2)this.handler.getDevice()).getConfig();
        result.put("cursor", value.cursor);
        result.put("blinking", value.blinking);
        return result;
    }

    public static Map<String, Object> brickletLCD16x2GetConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLCD16x2Actions) {
            return ((BrickletLCD16x2Actions) actions).brickletLCD16x2GetConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD16x2Actions class.");
        }
    }

    @RuleAction(label = "Is Button Pressed")
    public @ActionOutput(name = "pressed", type="boolean")
           Map<String, Object> brickletLCD16x2IsButtonPressed(
            @ActionInput(name = "button") int button) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickletLCD16x2)this.handler.getDevice()).isButtonPressed(button);
        result.put("pressed", value);
        return result;
    }

    public static Map<String, Object> brickletLCD16x2IsButtonPressed(@Nullable ThingActions actions, int button) throws TinkerforgeException {
        if (actions instanceof BrickletLCD16x2Actions) {
            return ((BrickletLCD16x2Actions) actions).brickletLCD16x2IsButtonPressed(button);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD16x2Actions class.");
        }
    }

    @RuleAction(label = "Set Custom Character")
    public void brickletLCD16x2SetCustomCharacter(
            @ActionInput(name = "index") int index,
            @ActionInput(name = "character") int[] character) throws TinkerforgeException {
        ((BrickletLCD16x2)this.handler.getDevice()).setCustomCharacter(index, character);
    }

    public static void brickletLCD16x2SetCustomCharacter(@Nullable ThingActions actions, int index, int[] character) throws TinkerforgeException {
        if (actions instanceof BrickletLCD16x2Actions) {
            ((BrickletLCD16x2Actions) actions).brickletLCD16x2SetCustomCharacter(index, character);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD16x2Actions class.");
        }
    }

    @RuleAction(label = "Get Custom Character")
    public @ActionOutput(name = "character", type="int[]")
           Map<String, Object> brickletLCD16x2GetCustomCharacter(
            @ActionInput(name = "index") int index) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int[] value = ((BrickletLCD16x2)this.handler.getDevice()).getCustomCharacter(index);
        result.put("character", value);
        return result;
    }

    public static Map<String, Object> brickletLCD16x2GetCustomCharacter(@Nullable ThingActions actions, int index) throws TinkerforgeException {
        if (actions instanceof BrickletLCD16x2Actions) {
            return ((BrickletLCD16x2Actions) actions).brickletLCD16x2GetCustomCharacter(index);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD16x2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletLCD16x2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletLCD16x2.Identity value = ((BrickletLCD16x2)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletLCD16x2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLCD16x2Actions) {
            return ((BrickletLCD16x2Actions) actions).brickletLCD16x2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD16x2Actions class.");
        }
    }
}
