package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletLoadCellActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Weight")
    public @ActionOutput(name = "weight", type="int")
           Map<String, Object> brickletLoadCellGetWeight(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletLoadCell)this.handler.getDevice()).getWeight();
        result.put("weight", value);
        return result;
    }

    public static Map<String, Object> brickletLoadCellGetWeight(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellActions) {
            return ((BrickletLoadCellActions) actions).brickletLoadCellGetWeight();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellActions class.");
        }
    }

    @RuleAction(label = "Tare")
    public void brickletLoadCellTare(
            ) throws TinkerforgeException {
        ((BrickletLoadCell)this.handler.getDevice()).tare();
    }

    public static void brickletLoadCellTare(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellActions) {
            ((BrickletLoadCellActions) actions).brickletLoadCellTare();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellActions class.");
        }
    }

    @RuleAction(label = "Get Moving Average")
    public @ActionOutput(name = "average", type="int")
           Map<String, Object> brickletLoadCellGetMovingAverage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletLoadCell)this.handler.getDevice()).getMovingAverage();
        result.put("average", value);
        return result;
    }

    public static Map<String, Object> brickletLoadCellGetMovingAverage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellActions) {
            return ((BrickletLoadCellActions) actions).brickletLoadCellGetMovingAverage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellActions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "rate", type="int")
           @ActionOutput(name = "gain", type="int")
           Map<String, Object> brickletLoadCellGetConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletLoadCell.Configuration value = ((BrickletLoadCell)this.handler.getDevice()).getConfiguration();
        result.put("rate", value.rate);
        result.put("gain", value.gain);
        return result;
    }

    public static Map<String, Object> brickletLoadCellGetConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellActions) {
            return ((BrickletLoadCellActions) actions).brickletLoadCellGetConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellActions class.");
        }
    }

    @RuleAction(label = "Is LED On")
    public @ActionOutput(name = "on", type="boolean")
           Map<String, Object> brickletLoadCellIsLEDOn(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickletLoadCell)this.handler.getDevice()).isLEDOn();
        result.put("on", value);
        return result;
    }

    public static Map<String, Object> brickletLoadCellIsLEDOn(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellActions) {
            return ((BrickletLoadCellActions) actions).brickletLoadCellIsLEDOn();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletLoadCellGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletLoadCell.Identity value = ((BrickletLoadCell)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletLoadCellGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellActions) {
            return ((BrickletLoadCellActions) actions).brickletLoadCellGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellActions class.");
        }
    }
}
