package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletOLED128x64Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Write")
    public void brickletOLED128x64Write(
            @ActionInput(name = "data") int[] data) throws TinkerforgeException {
        ((BrickletOLED128x64)this.handler.getDevice()).write(data);
    }

    public static void brickletOLED128x64Write(@Nullable ThingActions actions, int[] data) throws TinkerforgeException {
        if (actions instanceof BrickletOLED128x64Actions) {
            ((BrickletOLED128x64Actions) actions).brickletOLED128x64Write(data);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED128x64Actions class.");
        }
    }

    @RuleAction(label = "New Window")
    public void brickletOLED128x64NewWindow(
            @ActionInput(name = "columnFrom") int columnFrom,
            @ActionInput(name = "columnTo") int columnTo,
            @ActionInput(name = "rowFrom") int rowFrom,
            @ActionInput(name = "rowTo") int rowTo) throws TinkerforgeException {
        ((BrickletOLED128x64)this.handler.getDevice()).newWindow(columnFrom, columnTo, rowFrom, rowTo);
    }

    public static void brickletOLED128x64NewWindow(@Nullable ThingActions actions, int columnFrom, int columnTo, int rowFrom, int rowTo) throws TinkerforgeException {
        if (actions instanceof BrickletOLED128x64Actions) {
            ((BrickletOLED128x64Actions) actions).brickletOLED128x64NewWindow(columnFrom, columnTo, rowFrom, rowTo);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED128x64Actions class.");
        }
    }

    @RuleAction(label = "Clear Display")
    public void brickletOLED128x64ClearDisplay(
            ) throws TinkerforgeException {
        ((BrickletOLED128x64)this.handler.getDevice()).clearDisplay();
    }

    public static void brickletOLED128x64ClearDisplay(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOLED128x64Actions) {
            ((BrickletOLED128x64Actions) actions).brickletOLED128x64ClearDisplay();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED128x64Actions class.");
        }
    }

    @RuleAction(label = "Write Line")
    public void brickletOLED128x64WriteLine(
            @ActionInput(name = "line") int line,
            @ActionInput(name = "position") int position,
            @ActionInput(name = "text") String text) throws TinkerforgeException {
        ((BrickletOLED128x64)this.handler.getDevice()).writeLine(line, position, text);
    }

    public static void brickletOLED128x64WriteLine(@Nullable ThingActions actions, int line, int position, String text) throws TinkerforgeException {
        if (actions instanceof BrickletOLED128x64Actions) {
            ((BrickletOLED128x64Actions) actions).brickletOLED128x64WriteLine(line, position, text);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED128x64Actions class.");
        }
    }

    @RuleAction(label = "Get Display Configuration")
    public @ActionOutput(name = "contrast", type="int")
           @ActionOutput(name = "invert", type="boolean")
           Map<String, Object> brickletOLED128x64GetDisplayConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletOLED128x64.DisplayConfiguration value = ((BrickletOLED128x64)this.handler.getDevice()).getDisplayConfiguration();
        result.put("contrast", value.contrast);
        result.put("invert", value.invert);
        return result;
    }

    public static Map<String, Object> brickletOLED128x64GetDisplayConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOLED128x64Actions) {
            return ((BrickletOLED128x64Actions) actions).brickletOLED128x64GetDisplayConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED128x64Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletOLED128x64GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletOLED128x64.Identity value = ((BrickletOLED128x64)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletOLED128x64GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOLED128x64Actions) {
            return ((BrickletOLED128x64Actions) actions).brickletOLED128x64GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED128x64Actions class.");
        }
    }
}
