package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletRemoteSwitchActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Switching State")
    public @ActionOutput(name = "state", type="int")
           Map<String, Object> brickletRemoteSwitchGetSwitchingState(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletRemoteSwitch)this.handler.getDevice()).getSwitchingState();
        result.put("state", value);
        return result;
    }

    public static Map<String, Object> brickletRemoteSwitchGetSwitchingState(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRemoteSwitchActions) {
            return ((BrickletRemoteSwitchActions) actions).brickletRemoteSwitchGetSwitchingState();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRemoteSwitchActions class.");
        }
    }

    @RuleAction(label = "Switch Socket A")
    public void brickletRemoteSwitchSwitchSocketA(
            @ActionInput(name = "houseCode") int houseCode,
            @ActionInput(name = "receiverCode") int receiverCode,
            @ActionInput(name = "switchTo") int switchTo) throws TinkerforgeException {
        ((BrickletRemoteSwitch)this.handler.getDevice()).switchSocketA(houseCode, receiverCode, switchTo);
    }

    public static void brickletRemoteSwitchSwitchSocketA(@Nullable ThingActions actions, int houseCode, int receiverCode, int switchTo) throws TinkerforgeException {
        if (actions instanceof BrickletRemoteSwitchActions) {
            ((BrickletRemoteSwitchActions) actions).brickletRemoteSwitchSwitchSocketA(houseCode, receiverCode, switchTo);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRemoteSwitchActions class.");
        }
    }

    @RuleAction(label = "Switch Socket B")
    public void brickletRemoteSwitchSwitchSocketB(
            @ActionInput(name = "address") long address,
            @ActionInput(name = "unit") int unit,
            @ActionInput(name = "switchTo") int switchTo) throws TinkerforgeException {
        ((BrickletRemoteSwitch)this.handler.getDevice()).switchSocketB(address, unit, switchTo);
    }

    public static void brickletRemoteSwitchSwitchSocketB(@Nullable ThingActions actions, long address, int unit, int switchTo) throws TinkerforgeException {
        if (actions instanceof BrickletRemoteSwitchActions) {
            ((BrickletRemoteSwitchActions) actions).brickletRemoteSwitchSwitchSocketB(address, unit, switchTo);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRemoteSwitchActions class.");
        }
    }

    @RuleAction(label = "Dim Socket B")
    public void brickletRemoteSwitchDimSocketB(
            @ActionInput(name = "address") long address,
            @ActionInput(name = "unit") int unit,
            @ActionInput(name = "dimValue") int dimValue) throws TinkerforgeException {
        ((BrickletRemoteSwitch)this.handler.getDevice()).dimSocketB(address, unit, dimValue);
    }

    public static void brickletRemoteSwitchDimSocketB(@Nullable ThingActions actions, long address, int unit, int dimValue) throws TinkerforgeException {
        if (actions instanceof BrickletRemoteSwitchActions) {
            ((BrickletRemoteSwitchActions) actions).brickletRemoteSwitchDimSocketB(address, unit, dimValue);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRemoteSwitchActions class.");
        }
    }

    @RuleAction(label = "Switch Socket C")
    public void brickletRemoteSwitchSwitchSocketC(
            @ActionInput(name = "systemCode") char systemCode,
            @ActionInput(name = "deviceCode") int deviceCode,
            @ActionInput(name = "switchTo") int switchTo) throws TinkerforgeException {
        ((BrickletRemoteSwitch)this.handler.getDevice()).switchSocketC(systemCode, deviceCode, switchTo);
    }

    public static void brickletRemoteSwitchSwitchSocketC(@Nullable ThingActions actions, char systemCode, int deviceCode, int switchTo) throws TinkerforgeException {
        if (actions instanceof BrickletRemoteSwitchActions) {
            ((BrickletRemoteSwitchActions) actions).brickletRemoteSwitchSwitchSocketC(systemCode, deviceCode, switchTo);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRemoteSwitchActions class.");
        }
    }

    @RuleAction(label = "Set Repeats")
    public void brickletRemoteSwitchSetRepeats(
            @ActionInput(name = "repeats") int repeats) throws TinkerforgeException {
        ((BrickletRemoteSwitch)this.handler.getDevice()).setRepeats(repeats);
    }

    public static void brickletRemoteSwitchSetRepeats(@Nullable ThingActions actions, int repeats) throws TinkerforgeException {
        if (actions instanceof BrickletRemoteSwitchActions) {
            ((BrickletRemoteSwitchActions) actions).brickletRemoteSwitchSetRepeats(repeats);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRemoteSwitchActions class.");
        }
    }

    @RuleAction(label = "Get Repeats")
    public @ActionOutput(name = "repeats", type="int")
           Map<String, Object> brickletRemoteSwitchGetRepeats(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletRemoteSwitch)this.handler.getDevice()).getRepeats();
        result.put("repeats", value);
        return result;
    }

    public static Map<String, Object> brickletRemoteSwitchGetRepeats(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRemoteSwitchActions) {
            return ((BrickletRemoteSwitchActions) actions).brickletRemoteSwitchGetRepeats();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRemoteSwitchActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletRemoteSwitchGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRemoteSwitch.Identity value = ((BrickletRemoteSwitch)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletRemoteSwitchGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRemoteSwitchActions) {
            return ((BrickletRemoteSwitchActions) actions).brickletRemoteSwitchGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRemoteSwitchActions class.");
        }
    }
}
