package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletRotaryEncoderActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Count")
    public @ActionOutput(name = "count", type="int")
           Map<String, Object> brickletRotaryEncoderGetCount(
            @ActionInput(name = "reset") boolean reset) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletRotaryEncoder)this.handler.getDevice()).getCount(reset);
        result.put("count", value);
        return result;
    }

    public static Map<String, Object> brickletRotaryEncoderGetCount(@Nullable ThingActions actions, boolean reset) throws TinkerforgeException {
        if (actions instanceof BrickletRotaryEncoderActions) {
            return ((BrickletRotaryEncoderActions) actions).brickletRotaryEncoderGetCount(reset);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRotaryEncoderActions class.");
        }
    }

    @RuleAction(label = "Is Pressed")
    public @ActionOutput(name = "pressed", type="boolean")
           Map<String, Object> brickletRotaryEncoderIsPressed(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickletRotaryEncoder)this.handler.getDevice()).isPressed();
        result.put("pressed", value);
        return result;
    }

    public static Map<String, Object> brickletRotaryEncoderIsPressed(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRotaryEncoderActions) {
            return ((BrickletRotaryEncoderActions) actions).brickletRotaryEncoderIsPressed();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRotaryEncoderActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletRotaryEncoderGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRotaryEncoder.Identity value = ((BrickletRotaryEncoder)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletRotaryEncoderGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRotaryEncoderActions) {
            return ((BrickletRotaryEncoderActions) actions).brickletRotaryEncoderGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRotaryEncoderActions class.");
        }
    }
}
