package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletTemperatureActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletTemperatureGetTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletTemperature)this.handler.getDevice()).getTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletTemperatureGetTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletTemperatureActions) {
            return ((BrickletTemperatureActions) actions).brickletTemperatureGetTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletTemperatureActions class.");
        }
    }

    @RuleAction(label = "Get I2C Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletTemperatureGetI2CMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletTemperature)this.handler.getDevice()).getI2CMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletTemperatureGetI2CMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletTemperatureActions) {
            return ((BrickletTemperatureActions) actions).brickletTemperatureGetI2CMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletTemperatureActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletTemperatureGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletTemperature.Identity value = ((BrickletTemperature)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletTemperatureGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletTemperatureActions) {
            return ((BrickletTemperatureActions) actions).brickletTemperatureGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletTemperatureActions class.");
        }
    }
}
