package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletTemperatureIRActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Ambient Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletTemperatureIRGetAmbientTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletTemperatureIR)this.handler.getDevice()).getAmbientTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletTemperatureIRGetAmbientTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletTemperatureIRActions) {
            return ((BrickletTemperatureIRActions) actions).brickletTemperatureIRGetAmbientTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletTemperatureIRActions class.");
        }
    }

    @RuleAction(label = "Get Object Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletTemperatureIRGetObjectTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletTemperatureIR)this.handler.getDevice()).getObjectTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletTemperatureIRGetObjectTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletTemperatureIRActions) {
            return ((BrickletTemperatureIRActions) actions).brickletTemperatureIRGetObjectTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletTemperatureIRActions class.");
        }
    }

    @RuleAction(label = "Get Emissivity")
    public @ActionOutput(name = "emissivity", type="int")
           Map<String, Object> brickletTemperatureIRGetEmissivity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletTemperatureIR)this.handler.getDevice()).getEmissivity();
        result.put("emissivity", value);
        return result;
    }

    public static Map<String, Object> brickletTemperatureIRGetEmissivity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletTemperatureIRActions) {
            return ((BrickletTemperatureIRActions) actions).brickletTemperatureIRGetEmissivity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletTemperatureIRActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletTemperatureIRGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletTemperatureIR.Identity value = ((BrickletTemperatureIR)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletTemperatureIRGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletTemperatureIRActions) {
            return ((BrickletTemperatureIRActions) actions).brickletTemperatureIRGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletTemperatureIRActions class.");
        }
    }
}
