package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletVoltageActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> brickletVoltageGetVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletVoltage)this.handler.getDevice()).getVoltage();
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickletVoltageGetVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletVoltageActions) {
            return ((BrickletVoltageActions) actions).brickletVoltageGetVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageActions class.");
        }
    }

    @RuleAction(label = "Get Analog Value")
    public @ActionOutput(name = "value", type="int")
           Map<String, Object> brickletVoltageGetAnalogValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletVoltage)this.handler.getDevice()).getAnalogValue();
        result.put("value", value);
        return result;
    }

    public static Map<String, Object> brickletVoltageGetAnalogValue(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletVoltageActions) {
            return ((BrickletVoltageActions) actions).brickletVoltageGetAnalogValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletVoltageGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletVoltage.Identity value = ((BrickletVoltage)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletVoltageGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletVoltageActions) {
            return ((BrickletVoltageActions) actions).brickletVoltageGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageActions class.");
        }
    }
}
